@@ -2,7 +2,7 @@
  * vout_xvideo.c: Xvideo video output display method
  *****************************************************************************
  * Copyright (C) 1998, 1999, 2000, 2001 VideoLAN
- * $Id: vout_xvideo.c,v 1.25 2001/08/05 15:32:46 gbazin Exp $
+ * $Id: vout_xvideo.c,v 1.28 2001/09/30 01:05:37 stef Exp $
  *
  * Authors: Shane Harper <shanegh@optusnet.com.au>
  *          Vincent Seguin <seguin@via.ecp.fr>
@@ -54,6 +54,7 @@
 #include <X11/extensions/XShm.h>
 #include <X11/extensions/Xv.h>
 #include <X11/extensions/Xvlib.h>
+#include <X11/extensions/dpms.h>
 
 #include "config.h"
 #include "common.h"
@@ -429,6 +430,7 @@ static int vout_Manage( vout_thread_t *p_vout )
             /* Update dimensions */
             p_vout->p_sys->i_window_width = xevent.xconfigure.width;
             p_vout->p_sys->i_window_height = xevent.xconfigure.height;
+//            p_vout->i_changes |= VOUT_SIZE_CHANGE;
         }
         /* MapNotify event: change window status and disable screen saver */
         else if( xevent.type == MapNotify)
@@ -627,11 +629,12 @@ static int vout_Manage( vout_thread_t *p_vout )
     {
         intf_DbgMsg( "vout: resizing window" );
         p_vout->i_changes &= ~VOUT_SIZE_CHANGE;
-        /* Nothing to do here...
-         * vout_Display() detects size changes of the image to be displayed and
-         * re-creates the XvImage.*/
-        intf_Msg( "vout: video display resized (%dx%d)",
-                  p_vout->i_width, p_vout->i_height );
+
+        p_vout->i_width = p_vout->p_sys->i_window_width;
+        p_vout->i_height = p_vout->p_sys->i_window_height;
+
+        intf_WarnMsg( 3, "vout: video display resized (%dx%d)",
+                      p_vout->i_width, p_vout->i_height );
     }
 
     /* Autohide Cursor */
@@ -797,15 +800,11 @@ static int XVideoCreateWindow( vout_thread_t *p_vout )
                                                       p_vout->p_sys->i_screen );
         p_vout->p_sys->i_window_height =  DisplayHeight( p_vout->p_sys->p_display,
                                                          p_vout->p_sys->i_screen );
-/*      p_vout->i_width =  p_vout->p_sys->i_window_width;
-        p_vout->i_height = p_vout->p_sys->i_window_height; */
     }
     else
     {
-        p_vout->p_sys->i_window_width =  main_GetIntVariable( VOUT_WIDTH_VAR,
-                                                       VOUT_WIDTH_DEFAULT );
-        p_vout->p_sys->i_window_height = main_GetIntVariable( VOUT_HEIGHT_VAR,
-                                                       VOUT_HEIGHT_DEFAULT );
+        p_vout->p_sys->i_window_width =  p_vout->i_width;
+        p_vout->p_sys->i_window_height = p_vout->i_height;
     }
 
     /* Prepare window manager hints and properties */
@@ -1060,6 +1059,8 @@ void XVideoEnableScreenSaver( vout_thread_t *p_vout )
                      p_vout->p_sys->i_ss_interval,
                      p_vout->p_sys->i_ss_blanking,
                      p_vout->p_sys->i_ss_exposure );
+
+    DPMSEnable( p_vout->p_sys->p_display );
 }
 
 /*****************************************************************************
@@ -1081,6 +1082,8 @@ void XVideoDisableScreenSaver( vout_thread_t *p_vout )
                      p_vout->p_sys->i_ss_interval,
                      p_vout->p_sys->i_ss_blanking,
                      p_vout->p_sys->i_ss_exposure );
+
+    DPMSDisable( p_vout->p_sys->p_display );
 }
 
 /*****************************************************************************
@@ -1109,58 +1112,35 @@ void X11ToggleMousePointer( vout_thread_t *p_vout )
  * used by the xvideo plugin, but others may want to use it. */
 static void XVideoOutputCoords( const picture_t *p_pic, const boolean_t scale,
                                 const int win_w, const int win_h,
-                                int *dx, int *dy, int *w, int *h)
+                                int *dx, int *dy, int *w, int *h )
 {
     if( !scale )
     {
-        *w = p_pic->i_width; *h = p_pic->i_height;
+        *w = p_pic->i_width;
+        *h = p_pic->i_height;
     }
     else
     {
-        *w = win_w;
+        *h = win_h;
         switch( p_pic->i_aspect_ratio )
         {
             case AR_3_4_PICTURE:
-                *h = win_w * 3 / 4;
+                *w = win_h * 4 / 3;
                 break;
 
             case AR_16_9_PICTURE:
-                *h = win_w * 9 / 16;
+                *w = win_h * 16 / 9;
                 break;
 
             case AR_221_1_PICTURE:
-                *h = win_w * 100 / 221;
+                *w = win_h * 221 / 100;
                 break;
 
             case AR_SQUARE_PICTURE:
             default:
-                *h = win_w * p_pic->i_height / p_pic->i_width;
+                *w = win_h * p_pic->i_width / p_pic->i_height;
                 break;
         }
-
-        if( *h > win_h )
-        {
-            *h = win_h;
-            switch( p_pic->i_aspect_ratio )
-            {
-                case AR_3_4_PICTURE:
-                    *w = win_h * 4 / 3;
-                    break;
-
-                case AR_16_9_PICTURE:
-                    *w = win_h * 16 / 9;
-                    break;
-
-                case AR_221_1_PICTURE:
-                    *w = win_h * 221 / 100;
-                    break;
-
-                case AR_SQUARE_PICTURE:
-                default:
-                    *w = win_h * p_pic->i_width / p_pic->i_height;
-                    break;
-            }
-        }
     }
 
     /* Set picture position */
@@ -1169,14 +1149,19 @@ static void XVideoOutputCoords( const picture_t *p_pic, const boolean_t scale,
 }
 
 
+/*****************************************************************************
+ * XVideoGetPort: get YUV12 port
+ *****************************************************************************
+ * 
+ *****************************************************************************/
 static int XVideoGetPort( Display *dpy )
 {
-    int            i, i_adaptors;
-    int            xv_port = -1;
-    XvAdaptorInfo *adaptor_info;
+    XvAdaptorInfo *p_adaptor;
+    int i_adaptor, i_num_adaptors, i_requested_adaptor;
+    int i_selected_port;
 
     switch( XvQueryAdaptors( dpy, DefaultRootWindow( dpy ),
-                             &i_adaptors, &adaptor_info ) )
+                             &i_num_adaptors, &p_adaptor ) )
     {
         case Success:
             break;
@@ -1194,52 +1179,137 @@ static int XVideoGetPort( Display *dpy )
             return( -1 );
     }
 
-    for( i=0; i < i_adaptors && xv_port == -1; ++i )
+    i_selected_port = -1;
+    i_requested_adaptor = main_GetIntVariable( VOUT_XVADAPTOR_VAR, -1 );
+
+    /* No special xv port has been requested so try all of them */
+    for( i_adaptor = 0; i_adaptor < i_num_adaptors; ++i_adaptor )
     {
-        if( ( adaptor_info[ i ].type & XvInputMask ) &&
-            ( adaptor_info[ i ].type & XvImageMask ) )
+        int i_port;
+
+        /* If we requested an adaptor and it's not this one, we aren't
+         * interested */
+        if( i_requested_adaptor != -1 && i_adaptor != i_requested_adaptor )
+	{
+            continue;
+	}
+
+	/* If the adaptor doesn't have the required properties, skip it */
+        if( !( p_adaptor[ i_adaptor ].type & XvInputMask ) ||
+            !( p_adaptor[ i_adaptor ].type & XvImageMask ) )
         {
-            /* check that port supports YUV12 planar format... */
-            int port = adaptor_info[ i ].base_id;
-            int i_num_formats, i;
-            XvImageFormatValues *imageFormats;
+            continue;
+	}
 
-            imageFormats = XvListImageFormats( dpy, port, &i_num_formats );
+        for( i_port = p_adaptor[i_adaptor].base_id;
+             i_port < p_adaptor[i_adaptor].base_id
+                       + p_adaptor[i_adaptor].num_ports;
+             i_port++ )
+        {
+            XvImageFormatValues *p_formats;
+            int i_format, i_num_formats;
 
-            for( i=0; i < i_num_formats && xv_port == -1; ++i )
+            /* If we already found a port, we aren't interested */
+            if( i_selected_port != -1 )
             {
-                if( imageFormats[ i ].id == GUID_YUV12_PLANAR )
+                continue;
+            }
+
+            /* Check that port supports YUV12 planar format... */
+            p_formats = XvListImageFormats( dpy, i_port, &i_num_formats );
+
+            for( i_format = 0; i_format < i_num_formats; i_format++ )
+            {
+                XvEncodingInfo  *p_enc;
+                int             i_enc, i_num_encodings;
+                XvAttribute     *p_attr;
+                int             i_attr, i_num_attributes;
+
+                if( p_formats[ i_format ].id != GUID_YUV12_PLANAR )
                 {
-                    xv_port = port;
+                    continue;
+                }
+
+                /* Found a matching port, print a description of this port */
+                i_selected_port = i_port;
+
+                intf_WarnMsg( 3, "vout: XVideoGetPort found adaptor %i port %i",
+                                 i_adaptor, i_port);
+                intf_WarnMsg( 3, "  image format 0x%x (%4.4s) %s supported",
+                                 p_formats[ i_format ].id,
+                                 (char *)&p_formats[ i_format ].id,
+                                 ( p_formats[ i_format ].format
+                                    == XvPacked ) ? "packed" : "planar" );
+
+                intf_WarnMsg( 4, " encoding list:" );
+
+                if( XvQueryEncodings( dpy, i_port, &i_num_encodings, &p_enc )
+                     != Success )
+                {
+                    intf_WarnMsg( 4, "  XvQueryEncodings failed" );
+                    continue;
+                }
+
+                for( i_enc = 0; i_enc < i_num_encodings; i_enc++ )
+                {
+                    intf_WarnMsg( 4, "  id=%ld, name=%s, size=%ldx%ld,"
+                                     " numerator=%d, denominator=%d",
+                                  p_enc[i_enc].encoding_id, p_enc[i_enc].name,
+                                  p_enc[i_enc].width, p_enc[i_enc].height,
+                                  p_enc[i_enc].rate.numerator,
+                                  p_enc[i_enc].rate.denominator );
+                }
+
+                if( p_enc != NULL )
+                {
+                    XvFreeEncodingInfo( p_enc );
+                }
+
+                intf_WarnMsg( 4, " attribute list:" );
+                p_attr = XvQueryPortAttributes( dpy, i_port,
+                                                &i_num_attributes );
+                for( i_attr = 0; i_attr < i_num_attributes; i_attr++ )
+                {
+                    intf_WarnMsg( 4,
+                          "  name=%s, flags=[%s%s ], min=%i, max=%i",
+                          p_attr[i_attr].name,
+                          (p_attr[i_attr].flags & XvGettable) ? " get" : "",
+                          (p_attr[i_attr].flags & XvSettable) ? " set" : "",
+                          p_attr[i_attr].min_value, p_attr[i_attr].max_value );
+                }
+
+                if( p_attr != NULL )
+                {
+                    XFree( p_attr );
                 }
             }
 
-            if( xv_port == -1 )
+            if( p_formats != NULL )
             {
-                intf_WarnMsg( 3, "vout: XVideo image input port %d "
-                        "does not support the YUV12 planar format which is "
-                        "currently required by the xvideo output plugin",
-                        port );
-            }
-
-            if( imageFormats )
-            {
-                XFree( imageFormats );
+                XFree( p_formats );
             }
         }
     }
 
-    if( i_adaptors > 0 )
+    if( i_num_adaptors > 0 )
     {
-        XvFreeAdaptorInfo(adaptor_info);
+        XvFreeAdaptorInfo( p_adaptor );
     }
 
-    if( xv_port == -1 )
+    if( i_selected_port == -1 )
     {
-        intf_WarnMsg( 3, "vout: no suitable Xvideo image input port" );
+        if( i_requested_adaptor == -1 )
+        {
+            intf_WarnMsg( 3, "vout: no XVideo port found supporting YUV12" );
+        }
+        else
+        {
+            intf_WarnMsg( 3, "vout: XVideo adaptor %i does not support YUV12",
+                             i_requested_adaptor );
+        }
     }
 
-    return( xv_port );
+    return( i_selected_port );
 }
 
 
@@ -1265,6 +1335,7 @@ static void XVideoDisplay( vout_thread_t *p_vout )
                         &i_dest_x, &i_dest_y,
                         &i_dest_width, &i_dest_height);
 
+
     XvShmPutImage( p_vout->p_sys->p_display, p_vout->p_sys->xv_port,
                    p_vout->p_sys->yuv_window, p_vout->p_sys->gc,
                    p_vout->p_sys->p_xvimage,
@@ -1274,11 +1345,24 @@ static void XVideoDisplay( vout_thread_t *p_vout )
                    0 /*dest_x*/, 0 /*dest_y*/, i_dest_width, i_dest_height,
                    False );
 
+    /* YUV window */
     XResizeWindow( p_vout->p_sys->p_display, p_vout->p_sys->yuv_window,
                    i_dest_width, i_dest_height );
     XMoveWindow( p_vout->p_sys->p_display, p_vout->p_sys->yuv_window,
                  i_dest_x, i_dest_y );
 
+    /* Root window */
+    if( ( ( i_dest_width != p_vout->p_sys->i_window_width ) ||
+          ( i_dest_height != p_vout->p_sys->i_window_height ) ) &&
+        ! p_vout->b_fullscreen )
+    {
+        p_vout->p_sys->i_window_width = i_dest_width;
+        p_vout->p_sys->i_window_height = i_dest_height;
+//        p_vout->i_changes |= VOUT_SIZE_CHANGE;
+        XResizeWindow( p_vout->p_sys->p_display, p_vout->p_sys->window,
+                       i_dest_width, i_dest_height );
+    }
+
     /* Send the order to the X server */
     XSync( p_vout->p_sys->p_display, False );
 }
@@ -1321,3 +1405,4 @@ static void XVideoSetAttribute( vout_thread_t *p_vout,
         XFree( p_attrib );
 }
 #endif
+
