@@ -4,7 +4,7 @@
  * and spawn threads.
  *****************************************************************************
  * Copyright (C) 1998, 1999, 2000 VideoLAN
- * $Id: main.c,v 1.111 2001/08/07 02:48:25 sam Exp $
+ * $Id: main.c,v 1.117 2001/10/03 15:10:55 sam Exp $
  *
  * Authors: Vincent Seguin <seguin@via.ecp.fr>
  *          Samuel Hocevar <sam@zoy.org>
@@ -115,7 +115,8 @@
 #define OPT_COLOR               164
 #define OPT_FULLSCREEN          165
 #define OPT_OVERLAY             166
-#define OPT_SMP                 167
+#define OPT_XVADAPTOR           167
+#define OPT_SMP                 168
 
 #define OPT_CHANNELS            170
 #define OPT_SERVER              171
@@ -133,6 +134,7 @@
 #define OPT_WARNING             191
 #define OPT_VERSION             192
 #define OPT_STDOUT              193
+#define OPT_STATS               194
 
 /* Usage fashion */
 #define USAGE                     0
@@ -163,6 +165,7 @@ static const struct option longopts[] =
     {   "intf",             1,          0,      'I' },
     {   "warning",          1,          0,      OPT_WARNING },
     {   "stdout",           1,          0,      OPT_STDOUT },
+    {   "stats",            0,          0,      OPT_STATS },
 
     /* Audio options */
     {   "noaudio",          0,          0,      OPT_NOAUDIO },
@@ -186,6 +189,7 @@ static const struct option longopts[] =
     {   "yuv",              1,          0,      OPT_YUV },
     {   "fullscreen",       0,          0,      OPT_FULLSCREEN },
     {   "overlay",          0,          0,      OPT_OVERLAY },
+    {   "xvadaptor",        1,          0,      OPT_XVADAPTOR },
     {   "smp",              1,          0,      OPT_SMP },
 
     /* DVD options */
@@ -273,22 +277,20 @@ int main( int i_argc, char *ppsz_argv[], char *ppsz_env[] )
      */
     p_main->i_cpu_capabilities = CPUCapabilities();
     
-#if defined( __pentium__ ) || defined( __pentiumpro__ )
-    if( ! TestCPU( CPU_CAPABILITY_586 ) )
-    {
-        fprintf( stderr, "error: this program needs a Pentium CPU,\n"
-                         "please try a version without Pentium support\n" );
-        return( 1 );
-    }
-#endif
-
     /*
      * System specific initialization code
      */
 #if defined( SYS_BEOS ) || defined( SYS_DARWIN )
     system_Init( &i_argc, ppsz_argv, ppsz_env );
+
 #elif defined( WIN32 )
     _fmode = _O_BINARY;   /* sets the default file-translation mode on Win32 */
+
+#elif defined( SYS_LINUX )
+#   ifdef DEBUG
+    /* Activate malloc checking routines to detect heap corruptions. */
+    main_PutIntVariable( "MALLOC_CHECK_", 2 );
+#   endif
 #endif
 
     /*
@@ -302,7 +304,7 @@ int main( int i_argc, char *ppsz_argv[], char *ppsz_env[] )
         return( errno );
     }
 
-    intf_MsgImm( COPYRIGHT_MESSAGE );
+    intf_MsgImm( COPYRIGHT_MESSAGE "\n" );
 
     /*
      * Read configuration
@@ -319,6 +321,30 @@ int main( int i_argc, char *ppsz_argv[], char *ppsz_env[] )
      */
     RedirectSTDOUT();
 
+    if( p_main->b_stats )
+    {
+        char          p_capabilities[200];
+        p_capabilities[0] = '\0';
+
+#define PRINT_CAPABILITY( capability, string )                              \
+        if( p_main->i_cpu_capabilities & capability )                       \
+        {                                                                   \
+            strncat( p_capabilities, string " ",                            \
+                     sizeof(p_capabilities) - strlen(p_capabilities) );     \
+            p_capabilities[sizeof(p_capabilities) - 1] = '\0';              \
+        }
+
+        PRINT_CAPABILITY( CPU_CAPABILITY_486, "486" );
+        PRINT_CAPABILITY( CPU_CAPABILITY_586, "586" );
+        PRINT_CAPABILITY( CPU_CAPABILITY_PPRO, "Pentium Pro" );
+        PRINT_CAPABILITY( CPU_CAPABILITY_MMX, "MMX" );
+        PRINT_CAPABILITY( CPU_CAPABILITY_3DNOW, "3DNow!" );
+        PRINT_CAPABILITY( CPU_CAPABILITY_MMXEXT, "MMXEXT" );
+        PRINT_CAPABILITY( CPU_CAPABILITY_SSE, "SSE" );
+        PRINT_CAPABILITY( CPU_CAPABILITY_ALTIVEC, "Altivec" );
+        intf_StatMsg("info: CPU has capabilities %s", p_capabilities );
+    }
+
     /*
      * Initialize playlist and get commandline files
      */
@@ -532,6 +558,7 @@ static int GetConfiguration( int *pi_argc, char *ppsz_argv[], char *ppsz_env[] )
     p_main->b_video     = 1;
 
     p_main->i_warning_level = 0;
+    p_main->b_stats = 0;
 
     p_main->p_channel = NULL;
 
@@ -639,6 +666,10 @@ static int GetConfiguration( int *pi_argc, char *ppsz_argv[], char *ppsz_env[] )
             main_PutPszVariable( INTF_STDOUT_VAR, optarg );
             break;
 
+        case OPT_STATS:
+            p_main->b_stats = 1;
+            break;
+
         /* Audio options */
         case OPT_NOAUDIO:                                       /* --noaudio */
             p_main->b_audio = 0;
@@ -690,6 +721,9 @@ static int GetConfiguration( int *pi_argc, char *ppsz_argv[], char *ppsz_env[] )
         case OPT_OVERLAY:                                       /* --overlay */
             main_PutIntVariable( VOUT_OVERLAY_VAR, 1 );
             break;
+        case OPT_XVADAPTOR:                                   /* --xvadaptor */
+            main_PutIntVariable( VOUT_XVADAPTOR_VAR, atoi(optarg) );
+            break;
         case OPT_MOTION:                                         /* --motion */
             main_PutPszVariable( MOTION_METHOD_VAR, optarg );
             break;
@@ -840,6 +874,7 @@ static void Usage( int i_fashion )
           "\n  -g, --grayscale                \tgrayscale output"
           "\n      --fullscreen               \tfullscreen output"
           "\n      --overlay                  \taccelerated display"
+          "\n      --xvadaptor <adaptor>      \tXVideo adaptor"
           "\n      --color                    \tcolor output"
           "\n      --motion <module>          \tmotion compensation method"
           "\n      --idct <module>            \tIDCT method"
@@ -894,6 +929,7 @@ static void Usage( int i_fashion )
         "\n  " VOUT_GRAYSCALE_VAR "={1|0}             \tgrayscale or color output"
         "\n  " VOUT_FULLSCREEN_VAR "={1|0}            \tfullscreen"
         "\n  " VOUT_OVERLAY_VAR "={1|0}               \toverlay"
+        "\n  " VOUT_XVADAPTOR_VAR "=<adaptor>         \tXVideo adaptor"
         "\n  " MOTION_METHOD_VAR "=<method name>      \tmotion compensation method"
         "\n  " IDCT_METHOD_VAR "=<method name>        \tIDCT method"
         "\n  " YUV_METHOD_VAR "=<method name>         \tYUV method"
@@ -1008,12 +1044,6 @@ static void InstructionSignalHandler( int i_signal )
      * to an interface having been destroyed */
 
     /* Acknowledge the signal received */
-    fprintf( stderr, "warning: extended instructions unsupported, "
-                     "some optimizations will be disabled\n" );
-#ifdef SYS_LINUX
-    fprintf( stderr, "upgrade to kernel 2.4.x to get rid of this warning\n" );
-#endif
-
     i_illegal = 1;
     
 #ifdef HAVE_SIGRELSE
@@ -1061,7 +1091,7 @@ static int CPUCapabilities( void )
     slot_name( hi.cpu_type, hi.cpu_subtype, &psz_name, &psz_subname );
     /* FIXME: need better way to detect newer proccessors.
      * could do strncmp(a,b,5), but that's real ugly */
-    if( strcmp(psz_name, "ppc7400") || strcmp(psz_name, "ppc7450") )
+    if( !strcmp(psz_name, "ppc7400") || !strcmp(psz_name, "ppc7450") )
     {
         i_capabilities |= CPU_CAPABILITY_ALTIVEC;
     }
@@ -1139,6 +1169,15 @@ static int CPUCapabilities( void )
         {
             i_capabilities |= CPU_CAPABILITY_SSE;
         }
+        else
+        {
+            fprintf( stderr, "warning: your OS doesn't have support for "
+                             "SSE instructions, "
+                             "some optimizations\nwill be disabled\n" );
+#ifdef SYS_LINUX
+            fprintf( stderr, "(you will need Linux kernel 2.4.x or later)\n" );
+#endif
+        }
     }
     
     /* test for additional capabilities */
@@ -1176,6 +1215,25 @@ static int CPUCapabilities( void )
     signal( SIGILL, NULL );     
     return( i_capabilities );
 
+#elif defined( __powerpc__ )
+    /* Test for Altivec */
+    signal( SIGILL, InstructionSignalHandler );
+
+    i_illegal = 0;
+    if( setjmp( env ) == 0 )
+    {
+        asm volatile ("mtspr 256,%0" : : "r" (-1));
+    }
+
+    if( i_illegal == 0 )
+    {
+        i_capabilities |= CPU_CAPABILITY_ALTIVEC;
+    }
+
+    signal( SIGILL, NULL );     
+
+    return( i_capabilities );
+
 #else
     /* default behaviour */
     return( i_capabilities );
